//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	fpu-spi.c
// @target	Microchip dsPIC C compiler
//
// Support routines for interfacing a dsPIC microcontroller to the
// uM-FPU V3 floating point coprocessor using an SPI interface.
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	August 29, 2008
//	- modified for Microchip dsPIC C compiler
//	July 2, 2008
//	- original version
//=============================================================================

#include "p30f3012.h"
#include "fpu.h"

//-------------------- SPI definitions ----------------------------------------
// *** change these definitions to match target configuration ***

#define FCY 29480000UL				// clock frequency: 29.48 MHz
#include <libpic30.h>

#define SPISTAT		SPI1STATbits	// SPI status register
#define SPICON		SPI1CONbits		// SPI control register
#define SPIBUF		SPI1BUF			// SPI data buffer

#define SPI_SCLK	PORTBbits.RB6	// SPI SCLK pin
#define SPI_SDI		PORTBbits.RB5	// SPI SDI pin
#define SPI_SDO		PORTBbits.RB4	// SPI SDO pin

#define SPI_PPRE	3				// SPI clock primary pre-scale
#define SPI_SPRE	2				// SPI clock secondary pre-scale

//=============================================================================

//
union {                         	// temporary storage
	float fval;
	long  lval;
	unsigned int  ival[2];
	unsigned char bval[4];
} u;

//------------------- fpu_reset -----------------------------------------------

unsigned char fpu_reset(void) {
    int i;

	// SPI setup
	SPISTAT.SPIEN = 0;				// disable SPI
	SPICON.PPRE = SPI_PPRE;			// set prescalers for 5 MHz (max)
	SPICON.SPRE = SPI_SPRE;
	SPICON.CKE = 1;					// SPI mode 0
	SPICON.MSTEN = 1;				// master mode
	SPISTAT.SPIROV = 0;				// clear overflow flag
	SPISTAT.SPIEN = 1;				// enable SPI

	for (i = 0; i < 10; i++)		// send reset sequence
		fpu_write(0xFF);
	fpu_write(0);
	__delay_ms(10);					// wait for 10 msec
	fpu_write(SYNC);				// send SYNC command
	fpu_readDelay();				// return synchronization character
	return fpu_read();
}

//------------------- fpu_wait ------------------------------------------------
void fpu_wait(void) {
    while (SPISTAT.SPITBF) ;		// wait until transmit buffer is ready
	__delay_us(2);					// delay for SPI transmission to complete
    while (SPI_SDI == 1) ;    		// wait until FPU is ready
}

//------------------- fpu_write -----------------------------------------------
void fpu_write(unsigned char bval) {
    while (SPISTAT.SPITBF) ;		// wait until transmit buffer is ready
	SPISTAT.SPIROV = 0;
	SPIBUF = bval;
}

void fpu_write2(unsigned char bval1, unsigned char bval2) {
    fpu_write(bval1);
    fpu_write(bval2);
}

void fpu_write3(unsigned char bval1, unsigned char bval2, unsigned char bval3) {
    fpu_write(bval1);
    fpu_write(bval2);
    fpu_write(bval3);
}

void fpu_write4(unsigned char bval1, unsigned char bval2, unsigned char bval3, unsigned char bval4) {
    fpu_write(bval1);
    fpu_write(bval2);
    fpu_write(bval3);
    fpu_write(bval4);
}

//------------------- fpu_writeWord -------------------------------------------
void fpu_writeWord(int wval) {
    u.ival[0] = wval;
    fpu_write(u.bval[1]);
    fpu_write(u.bval[0]);
}

//------------------- fpu_writeLong -------------------------------------------
void fpu_writeLong(long lval) {
    u.lval = lval;
    fpu_write(u.bval[3]);
    fpu_write(u.bval[2]);
    fpu_write(u.bval[1]);
    fpu_write(u.bval[0]);
}

//------------------- fpu_writeFloat ------------------------------------------
void fpu_writeFloat(float fval) {
    u.fval = fval;
    fpu_write(u.bval[3]);
    fpu_write(u.bval[2]);
    fpu_write(u.bval[1]);
    fpu_write(u.bval[0]);
}

//------------------- fpu_writeString -----------------------------------------
void fpu_writeString(char *s) {
    while (*s) fpu_write(*s++);
    fpu_write(0);
}

//------------------- fpu_read ------------------------------------------------
unsigned char fpu_read(void) {

    __delay_us(1);					// extra read delay
	u.bval[0] = SPIBUF;				// clear buffer
    while (SPISTAT.SPITBF) ;		// wait until transmit buffer is ready
	SPISTAT.SPIROV = 0;				// clear overflow flag
	SPIBUF = 0;						// send zero byte
	while (!SPISTAT.SPIRBF) ;		// wait until byte received
    return SPIBUF;					// return the byte
}

//------------------- fpu_readByte --------------------------------------------
unsigned char fpu_readByte(void) {

	fpu_readDelay();
    return fpu_read();
}

//------------------- fpu_readWord --------------------------------------------

int fpu_readWord(void) {
    u.bval[1] = fpu_readByte();
    u.bval[0] = fpu_read();
    return u.ival[0];
}

//------------------- fpu_readLong --------------------------------------------
long fpu_readLong(void) {
    u.bval[3] = fpu_readByte();
    u.bval[2] = fpu_read();
    u.bval[1] = fpu_read();
    u.bval[0] = fpu_read();
    return u.lval;
}

//------------------- fpu_readFloat -------------------------------------------
float fpu_readFloat(void) {
    u.bval[3] = fpu_readByte();
    u.bval[2] = fpu_read();
    u.bval[1] = fpu_read();
    u.bval[0] = fpu_read();
    return u.fval;
}

//------------------- fpu_readString ------------------------------------------
char *fpu_readString(char *s) {
    char *sb, c;

	sb = s;
    fpu_wait();
    fpu_write(READSTR);
    fpu_readDelay();
    while (1) {
        c = fpu_read();
        if (!c) break;
        *s++ = c;
    }
    *s = '\0';
	return sb;
}

//------------------- fpu_readStatus ------------------------------------------

unsigned char fpu_readStatus(void) {
    fpu_wait();
    fpu_write(READSTATUS);
    return fpu_readByte();
}

//------------------- fpu_readDelay -------------------------------------------
void fpu_readDelay(void) {
    __delay_us(15);
}

