//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	sample.c
// @target	Microchip dsPIC C compiler
//
// This program provides an example of using the uM-FPU V3 floating point
// coprocessor with the dsPIC using an SPI interface. This demo takes an
// integer value representing the diameter of a circle in centimeters,
// converts the value to inches, then calculates the circumference and area.
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	August 26, 2008
//	- updated for Microchip dsPIC C compiler
//	November 17, 2006
//	- original version
//=============================================================================

// select device and set configuration bits
#include <p30f3012.h>
_FOSC(CSW_FSCM_OFF & FRC_PLL16);
_FWDT(WDT_OFF);
_FICD(ICS_PGD3);

// set clock frequency
#define FCY 29480000UL				// clock frequency: 29.48 MHz
#include <libpic30.h>

#include <stdio.h>
#include "fpu.h"

//-------------------- uM-FPU register definitions ----------------------------

#define DiameterIn		1			// diameter in inches
#define Circumference	2			// circumference
#define Area			3			// area
#define Pi				4			// constant pi

//------------------ main -----------------------------------------------------

int main(void) {
	int diameterCm;

	// UART setup
	__C30_UART = 1;				// select UART1 for stdio
	U1BRG = 95;					// 19,200 baud @ 29.48 MHz
	U1MODEbits.ALTIO = 1;		// alternate I/O port
	U1MODEbits.UARTEN = 1;		// enable UART

	// print program name
	printf("\r\n\r\nConversion Example\r\n------------------\r\n");

	//------------------------------------
	// Reset the uM-FPU and print version.
	//------------------------------------
	if (fpu_reset() == SYNC_CHAR) {
		print_version();
		print_CRLF();
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		while(1) ;  			// stop if FPU reset fails
	}

	//-----------------------------
	// Load constant for later use.
	//-----------------------------
	fpu_write4(SELECTA, Pi, LOADPI, FSET0);

	//---------------------------------------------------------------------------
	// Get diameter in centimeters.  The value would typically come from a sensor
	// reading, but in this example an assumed value of 25 is used.
	//---------------------------------------------------------------------------
	diameterCm = 25;
	printf("Diameter (cm):       %i\r\n", diameterCm);

	//------------------------------
	// convert inches to centimeters
	//------------------------------
	fpu_write4(SELECTA, DiameterIn, FSETI, diameterCm);
	fpu_write2(FCNV, 5);
	printf("Diameter (in.):      ");
	print_float(0);
	print_CRLF();

	//------------------------------
	// circumference = diameter * pi
	//------------------------------
	fpu_write4(SELECTA, Circumference, FSET, DiameterIn);
	fpu_write2(FMUL, Pi);
	printf("Circumference (in.): ");
	print_float(0);
	print_CRLF();

	//----------------------------
	// area = (diameter / 2)^2 * pi
 	//----------------------------
	fpu_write4(SELECTA, Area, FSET, DiameterIn);
	fpu_write2(FDIVI, 2);
	fpu_write4(FMUL, Area, FMUL, Pi);
	printf("Area (sq.in.):       ");
	print_float(0);
	print_CRLF();

	printf("\r\nDone.\r\n");
	while(1) ;
}
